/*

File: Ebay.js

JavaScript logic for EbayAddict widget

Version: 1.0

 Copyright 2006 Scott Stickeler. All rights reserved.


*/

var scrollArea, scrollbar;

// load - Called when the widget it first loaded to initialize the widget

function load ()
{
    if (!window.widget)
    {       
      show ();
    } 
    else
    {
    	scrollbar = new AppleVerticalScrollbar(document.getElementById("myScrollBar"));
	    scrollArea = new AppleScrollArea(document.getElementById("contents"), scrollbar);
	    scrollArea.scrollsHorizontally = false;
	    scrollArea.singlepressScrollPixels = 15;
	    
      loadParameters(); 
      whiteInfoButton = new AppleInfoButton(document.getElementById("infoButton"), document.getElementById("front"), "white", "white", showPrefs);
	    glassDoneButton = new AppleGlassButton(document.getElementById("doneButton"), "Cancel", hidePrefs);
    }
}


var last_updated = "";
var whiteInfoButton;
var glassDoneButton;
var itemArray = new Array;
var numItems=0;
var numItemsReceived=0;
var url = "http://cgi.ebay.com/ws/eBayISAPI.dll?ViewItem&item=";

// show - Show the main widget window

function show ()
{
  window.resizeTo(595,248);
}

// requestItem - Send an HTTP request to retrieve the item info

function requestItem( item )
{
  alert("Requesting Id "+itemArray[item].id);  
  if (itemArray[item].xml_request != null)
	{
		itemArray[item].xml_request.abort();
		itemArray[item].xml_request = null;
	}
	
	itemArray[item].xml_request = new XMLHttpRequest();

	itemArray[item].xml_request.onload = function(e) {xml_loaded(e, item);}
	alert("Getting " + url + itemArray[item].id );
	itemArray[item].xml_request.open("GET", url + itemArray[item].id);
	itemArray[item].xml_request.send("");
}

// focused is called when the widget gets key focus

function focused()
{
	document.getElementById('contents').style.color = 'white'; 
	scrollArea.focus();
}

// blurred is called when the widget looses key focus

function blurred()
{
	document.getElementById('contents').style.color = 'gray'; 
	scrollArea.blur();
}

// Here we register for some widget events

if(window.widget)
{
  widget.onshow = show;
	window.onfocus = focused;
	window.onblur = blurred;
}	

// showPrefs() is called when the preferences flipper is clicked upon.  It freezes the front of the widget,
// hides the front div, unhides the back div, and then flips the widget over.

function showPrefs()
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	
	if (window.widget)
		widget.prepareForTransition("ToBack");		// freezes the widget so that you can change it without the user noticing
	
	front.style.display="none";		// hide the front
	back.style.display="block";		// show the back
	
	if (window.widget)
		setTimeout ('widget.performTransition();', 0);		// and flip the widget over	

}

// hidePrefs() is called by the done button on the back side of the widget.  It performs the opposite transition
// as showPrefs() does.

function hidePrefs()
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	
	if (window.widget)
		widget.prepareForTransition("ToFront");		// freezes the widget and prepares it for the flip back to the front
	
	back.style.display="none";			// hide the back
	front.style.display="block";		// show the front
	
	if (window.widget)
		setTimeout ('widget.performTransition();', 0);		// and flip the widget back to the front
}

// When the add button is clicked upon or clicked on, moused out, and moused up,
// it needs to be replaced by the unactive version of the button.

function buttonUpOut()
{
	document.getElementById("addButton").src = "Images/Button.png";
}

// addItem() - Called when the user enters a new item ID. Adds ID to the item list, saves it in the preference
//             file and requests the item information
function addItem()
{
  var item_id;
  
  document.getElementById("addButton").src = "Images/Button_down.png";
  item_id = document.getElementById("itemId").value;
  
  if( item_id == "" )
    return;
  
  hidePrefs();
  itemArray[numItems] = {id:item_id,desc:null,price:null,timeleft:null,xml_request:null,is_valid:0};
  alert("Added Item Id "+itemArray[numItems].id);
  numItems++;
  
  saveParameters();
  updateItems();
}

// updateItems() - Refresh the item list with the latest information

function updateItems()
{
  var item_cnt;
  var contents = document.getElementById('contents');
  
  numItemsReceived = 0;
	
	alert("Updating Item Status");
  for( item_cnt = 0; item_cnt < numItems; item_cnt++ )
  {
    requestItem( item_cnt );
  }
  

  setTimeout("updateItems()",60000); // Update once per minute
}

// saveParameters() - Saves item list to preference file

function saveParameters()
{
  if(window.widget) 
  { 
    var itemList = "";
    var item_cnt;
    
    for( item_cnt = 0; item_cnt < itemArray.length; item_cnt++ )
    {
      itemList += itemArray[item_cnt].id;
      
      if( item_cnt != itemArray.length - 1 )
      {
        itemList += ",";
      }
    }
    
    widget.setPreferenceForKey(itemList,"itemList"); 
  } 
}

// loadParameters() - Loads item list from parameter file

function loadParameters()
{
  if(window.widget) 
  { 
    var itemList = widget.preferenceForKey("itemList");
    
    if (itemList && itemList.length > 0) 
    { 
      var itemIds = itemList.split(",");
      
      if( itemIds == null )
        return;
        
      var item_cnt;
      
      for( item_cnt = 0; item_cnt < itemIds.length; item_cnt++ )
      {
        itemArray[item_cnt] = {id:itemIds[item_cnt],desc:null,price:null,timeleft:null,xml_request:null};
        alert("Added Item Id "+itemArray[numItems].id);
        numItems++;
      }
      updateItems(); 
    } 
  } 
}

//---------------------------------------------------------------------------------------------
//
// findChild - scan the children of a given DOM element for a node matching nodeName; much more
//				efficient than the standard DOM methods (getElementsByTagName, etc) if you know
//				what you're looking for
//
//----------------------------------------------------------------------------------------------
function findChild (element, nodeName)
{
	var child;
	
	for (child = element.firstChild; child != null; child = child.nextSibling)
	{
		if (child.nodeName == nodeName)
			return child;
	}
	
	return null;
}

// parseXMLResponse() - Process item information XML response

function parseXMLResponse( item )
{
  var resp = itemArray[item].xml_request.responseText + "";
  
	// Set RegEx Patterns
	var matchTitle = /<title>eBay.*?: (.+)\(.*?\)<\/title>/;
	//var matchTimeLeft = /End time:.*?\<nobr\>(.*?)\<\/nobr\>/;
	var matchTimeLeft = /End time:(.*)/;
	var matchValid = /<title>eBay/;
	var matchStatus = /Ended:/;

	// Internationlization
	var matchPriceUS = /<b>US.\$([0-9,]{0,8}[.][0-9]{0,2})/;
	var matchPriceUK = /<b>GBP.([0-9,]{0,8}[.][0-9]{0,2})/;
	var matchPriceAU = /<b>AU.\$([0-9,]{0,8}[.][0-9]{0,2})/;
	var matchPriceEU = /EUR.([0-9,]{0,8}[.][0-9]{0,2})/;
	var matchPriceIN = /<b>INR.([0-9,]{0,8}[.][0-9]{0,2})/;
	var matchPriceCA = /<b>C.\$([0-9,]{0,8}[.][0-9]{0,2})/;
	var matchPricePH = /<b>PHP.([0-9,]{0,8}[.][0-9]{0,2})/;
	var matchPriceFailSafe = /([0-9,]{1,8}[.][0-9]{2})/;
	
	if ( resp )
	{		
		// Check if item is valid
		if(resp.match(matchValid))
		{
			// Check Status
			if(resp.match(matchStatus))
			{
				 itemArray[item].timeleft = "Ended";
			} 
			else 
			{
				if( resp.match(matchTimeLeft) )
				{
				  //alert( resp.match(matchTimeLeft) );
				  //alert("Time: " + resp.match(matchTimeLeft)[1]);
				  itemArray[item].timeleft = timeFormat(resp.match(matchTimeLeft)[1]);
				}
				else
				{
					itemArray[item].timeleft = 0;
				}
			}

      alert("Time Left: " + itemArray[item].timeleft );
      
			// Match Price
			// Match US
			if(resp.match(matchPriceUS))
			{
				itemArray[item].price = "$" + resp.match(matchPriceUS)[1];
			// Match UK
			} 
			else if(resp.match(matchPriceUK)) 
			{
				itemArray[item].price = "GBP" + resp.match(matchPriceUK)[1];
			// Match EU
			} 
			else if(resp.match(matchPriceEU)) 
			{
				itemArray[item].price = "EU" + resp.match(matchPriceEU)[1];
			// Match AU
			} 
			else if(resp.match(matchPriceAU)) 
			{
				itemArray[item].price = "$" + resp.match(matchPriceAU)[1];
			// Match IN
			} 
			else if(resp.match(matchPriceIN)) 
			{
				itemArray[item].price = "IN" + resp.match(matchPriceIN)[1];
			// Match CA
			} 
			else if(resp.match(matchPriceCA)) 
			{
				itemArray[item].price = "$" + resp.match(matchPriceCA)[1];
			// Match PH
			} 
			else if(resp.match(matchPricePH)) 
			{
				itemArray[item].price = "PHP" + resp.match(matchPricePH)[1];
			// One Last Try
			} 
			else if(resp.match(matchPriceFailSafe)) 
			{
				itemArray[item].price = "$" + resp.match(matchPriceFailSafe)[1];
			// Eh, no luck
			} 
			else 
			{
			  alert("No Price Found");
			  itemArray[item].price = "Unknown";
			}

      alert("Price: " + itemArray[item].price );
      
      // Match Title
			itemArray[item].desc = resp.match(matchTitle)[1];
			alert("Desc: " + itemArray[item].desc );
			
		}else
		{
			alert("Invalid Auction");
			return 0;
		}
		
		return 1;
	}
	else
	{
	  return 0;
	}
}

//---------------------------------------------------------------------------------------------
//
// xml_loaded - extract the content of Apple RSS Hot News feed and place the items data into a
//			    results array: extract the title, link and publication date for each item. 
//
//----------------------------------------------------------------------------------------------

function xml_loaded (e, item) 
{ 
	var time_left;
	
	//alert(itemArray[item].xml_request.responseText);
	
	numItemsReceived++;
	
	alert("Response received for item "+itemArray[item].id);
	
	if( parseXMLResponse(item) )
	{
	  itemArray[item].is_valid = 1;
	}
	
	if( numItems == numItemsReceived )
	{
	  redrawItemList();
	}
	
	itemArray[item].xml_request = null;
}

// redrawItemList() - Draw item information on the widget content pane

function redrawItemList()
{
  var contents = document.getElementById('contents');
	var item_cnt;
	var date = new Date();
	var even = 0;
	  
  while (contents.hasChildNodes())
  {
		contents.removeChild(contents.firstChild);
	}
	  
	// Sort array by item ID
	itemArray.sort(function(a,b){return( a.id - b.id )});
	  
	for( item_cnt = 0; item_cnt < numItems; item_cnt++ )
	{
	  var row = document.createElement ('div');
	  row.setAttribute ('class', 'row ' + ( even ? 'dark' : 'light' ) );
	  
	  if( even )
	  {
	    even = 0;
	  }
	  else
	  {
	    even = 1;
	  }
	  
	  var item_div = document.createElement ('div');
		item_div.innerText = itemArray[item_cnt].id;
		item_div.setAttribute ('class', 'item');
    item_div.setAttribute ('onclick', 'clickOnItem (event, ' + itemArray[item_cnt].id + ');');

	  row.appendChild (item_div);
	 
	  if( itemArray[item_cnt].is_valid )
	  {
	    var desc_div = document.createElement ('div');
		  desc_div.innerText = itemArray[item_cnt].desc;
		  desc_div.setAttribute ('class', 'description');

	    row.appendChild (desc_div);
	  
	    var price_div = document.createElement ('div');
		  price_div.innerText = itemArray[item_cnt].price;
		  price_div.setAttribute ('class', 'price');

	    row.appendChild (price_div);
	  
	 	  var timeleft_div = document.createElement ('div');

		  timeleft_div.innerText = itemArray[item_cnt].timeleft;
		  timeleft_div.setAttribute ('class', 'timeleft');

	    row.appendChild (timeleft_div);
	  }
	  else
	  {
	    var desc_div = document.createElement ('div');
		  desc_div.innerText = "Item Not Found";
		  desc_div.setAttribute ('class', 'description');

	    row.appendChild (desc_div);
	  }
	  
	  // Delete Button
	  var delete_button = document.createElement('span');

	  delete_button.setAttribute('onmousedown','deleteButtonDown(' + item_cnt + ');');
	  delete_button.setAttribute('onmouseup','deleteButtonUp(' + item_cnt + ');');
	  delete_button.setAttribute('onmouseout','deleteButtonUp(' + item_cnt + ');');

	  var delete_button_img = document.createElement('img');
	 	    
  	delete_button_img.setAttribute('class','deletebutton');
	 	delete_button_img.setAttribute('src','Images/delete.png');  
	 	delete_button.appendChild( delete_button_img );
	
	  row.appendChild (delete_button);
	    
		contents.appendChild(row);

		var hours = date.getHours();
		var minutes = date.getMinutes();
		var seconds = date.getSeconds();
		
		var am_pm = "AM";
			
		if( hours > 12 )
		{
			hours -= 12;
		  am_pm = "PM";
		}
		else
		if( hours == 12 )
		{
		  am_pm = "PM";
		}
				
	  if( hours < 10 )
	  {
	    hours = "0" + hours;
	  }
	  
	  if( minutes < 10 )
	  {
	    minutes = "0" + minutes;
	  }
	  
	  if( seconds < 10 )
	  {
	    seconds = "0" + seconds;
	  }
	  
	  last_updated = hours + ":" +
		               minutes + ":" +
		               seconds + " " + am_pm;
		               
	  var update_time = document.getElementById('updatetime');
	
	  update_time.innerText = "Last Update: " + last_updated;
	}
	scrollArea.refresh();
}

// timeFormat() - Convert time from web page into the format we want

function timeFormat(t)
{
	var d = 0;
	var h = 0;
	var m = 0;
	var s = 0;
	var time = "";
	var matchDays = /([0-9]{0,2}) day/;
	var matchHours = /([0-9]{0,2}) hour/;
	var matchMinutes = /([0-9]{0,2}) min/;
	var matchSeconds = /([0-9]{0,2}) sec/;
	
	if(t.match(matchDays)){
		d = t.match(matchDays)[1];
		time = d + "d ";
	}
	if(t.match(matchHours)){
		h = t.match(matchHours)[1];
		time = time + h + "h ";
	}
	if(t.match(matchMinutes)){
		m = t.match(matchMinutes)[1];
		time = time + m + "m ";
	}
	if(t.match(matchSeconds)){
		s = t.match(matchSeconds)[1];
		time = time + s + "s ";
	}
	return time;
}


// deleteButtonDown() - Called when an item's delete button is pressed

function deleteButtonDown( item_index )
{
  alert("Button Down" + item_index );
  itemArray.splice( item_index, 1 );
  numItems--;
  saveParameters();
  redrawItemList();
}

// deleteButtonUp() - Called when an item's delete button is released

function deleteButtonUp( item_index )
{
  alert("Button Up" + item_index );
}

function clickOnItem( event, id )
{
	if (window.widget)
	{
		widget.openURL ( url + id );
	}
}